% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : CRB computation
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%      aucune

function [CRB_sigma2_known,CRB_sigma2_unknown,CRB_slope_sigma2_known,CRB_slope_sigma2_unknown]=crb_whittle_slope_plus_noise(gamma,alpha,sigma2,fnorm_begin_slope)

%+++++ Fisher information matrix +++++
F = zeros(3,3);
%(gamma,gamma) element
fun_11 = @(x,alpha,fnorm_begin_slope) 1/((1+gamma)^2) * (x < fnorm_begin_slope)...
    + ((fnorm_begin_slope^(2*alpha))./((gamma*fnorm_begin_slope^alpha + x.^alpha).^2)) .* (x >= fnorm_begin_slope);
F(1,1) = integral(@(x)fun_11(x,alpha,fnorm_begin_slope),0,0.5,'RelTol',0,'AbsTol',1e-12);

%(gamma,alpha) element
fun_12 = @(x,alpha,fnorm_begin_slope) 0 * (x < fnorm_begin_slope)...
    + gamma * ((fnorm_begin_slope^(2*alpha))./((gamma*fnorm_begin_slope^alpha + x.^alpha).^2))...
    .* log(fnorm_begin_slope./x) .* (x >= fnorm_begin_slope);
F(1,2) = integral(@(x)fun_12(x,alpha,fnorm_begin_slope),0,0.5,'RelTol',0,'AbsTol',1e-12);
F(2,1) = F(1,2);

%(gamma,sigma2) element
fun_13 = @(x,alpha,fnorm_begin_slope) 1/((1+gamma)*sigma2) * (x < fnorm_begin_slope)...
    + 1/sigma2 * (fnorm_begin_slope^alpha)./(gamma*fnorm_begin_slope^alpha + x.^alpha) .* (x >= fnorm_begin_slope);
F(1,3) = integral(@(x)fun_13(x,alpha,fnorm_begin_slope),0,0.5,'RelTol',0,'AbsTol',1e-12);
F(3,1) = F(1,3);

%(alpha,alpha) element
fun_22 = @(x,alpha,fnorm_begin_slope) 0 * (x < fnorm_begin_slope)...
    + (gamma^2) * ((fnorm_begin_slope^(2*alpha))./((gamma*fnorm_begin_slope^alpha + x.^alpha).^2))...
    .* (log(fnorm_begin_slope./x).^2) .* (x >= fnorm_begin_slope);
F(2,2) = integral(@(x)fun_22(x,alpha,fnorm_begin_slope),0,0.5,'RelTol',0,'AbsTol',1e-12);

%(alpha,sigma2) element
fun_23 = @(x,alpha,fnorm_begin_slope) 0 * (x < fnorm_begin_slope)...
    + gamma / sigma2 * (fnorm_begin_slope^alpha)./(gamma*fnorm_begin_slope^alpha + x.^alpha)...
    .* log(fnorm_begin_slope./x) .* (x >= fnorm_begin_slope);
F(2,3) = integral(@(x)fun_23(x,alpha,fnorm_begin_slope),0,0.5,'RelTol',0,'AbsTol',1e-12);
F(3,2) = F(2,3);


%(sigma2,sigma2) element
% fun_33 = @(x) 1/ (sigma2^2);
% F(3,3) = integral(@(x)fun_33(x),0,0.5,'RelTol',0,'AbsTol',1e-12)
F(3,3) = 0.5 / (sigma2^2);

%++++++++++++++++++++++++++

%+++++ Cramer-Rao bounds +++++
CRB_sigma2_unknown = F\eye(3);
CRB_slope_sigma2_unknown = CRB_sigma2_unknown(2,2);
CRB_sigma2_known = F(1:2,1:2)\eye(2);
CRB_slope_sigma2_known = CRB_sigma2_known(2,2);
