% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : Evaluation de la pente et du plancher de bruit par la
% methode : " Overall model 3 params" : 
% Estimation of gamma, alpha, sigma2
%               f1 (freq of beginning of the slope) assumed to be known
% Fitting of the model on a given frequency interval (freqLimits_norm, en
% entr?e de la fonction)
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%       estime_pre_process
%       get_index_norm
%       slope_plus_noise_gamma
% --------------------
% En entree :   SpectreSLA : spectre estime
%               freq_norm: vecteur des frequences normalisees
%               freqLimits_norm : zone frequentielle du fitting
%               Slope_norm : f1 in the paper, frequency between plateau and beginning of the slope
%               freq_init_norm : zone frequentielle de linearite du spectre log-log
% --------------------
% En sortie :   pentespectre : pente estimee
%               sigmabruit : plancher de bruit en energie
%               Sth : Spectre du modele
%               theta_fit : result of the fitting : gamma pente sigma2
% --------------------
function [pentespectre, sigmabruit,Sth,theta_fit] = estime_noise_gamma(SpectreSLA,freq_norm,freqLimits_norm,Slope_norm,freq_init_norm)

[log_Sx, theta_ini, tab_log_fnorm] = estime_pre_process(SpectreSLA,freq_norm,freq_init_norm);
% may 2109 : Co changes name log_Sx_ARWARP en log_Sx to avoid ambiguities
fnorm_begin_slope = Slope_norm;    
%Define the frequency band over which fitting of log \hat{S}_x(f) is performed
%Fitting over the interval [fnorm_begin_fit fnorm_end_fit]
ind_linreg =  get_index_norm(freqLimits_norm,tab_log_fnorm');

%Fitting of log(S_{ARWARP}(f)) over [fnorm_begin_fit,fnorm_end_fit]
options_lsqcurvefit = optimoptions(@lsqcurvefit,'Jacobian','on','Display','off');
obj_fun=@(theta,u)slope_plus_noise_gamma(theta,u,fnorm_begin_slope);
[theta_fit,~] = lsqcurvefit(obj_fun,theta_ini,tab_log_fnorm(ind_linreg),log_Sx(ind_linreg),[],[],options_lsqcurvefit);
theta_fit = real(theta_fit);
pentespectre = theta_fit(2);
sigmabruit = theta_fit(3);
%Compute fitted spectrum
Sth = exp(slope_plus_noise_gamma(theta_fit,tab_log_fnorm,fnorm_begin_slope))'; 
% rajout Co 2019-01-10 
% disp(['en sortie de Estime noise Gamma : gamma = ' num2str(theta_fit(1)) ' alpha = ' num2str(theta_fit(2)) ' sigma2 = ' num2str(theta_fit(3)) ' fnormbeginslope = ' num2str(fnorm_begin_slope) ]);
% save('echelle.mat','tab_log_fnorm','theta_fit','fnorm_begin_slope');
% fin rajout Co 2019-01-10
end

