% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : Calcul pour l'initialisation des parametres avant
% utilisation de lsqcurvefit
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%      get_index_norm
% --------------------
% En entree :   SpectreSLA : spectre estime sur lequel on fait le fitting
%               freq_norm: vecteur des frequences normalisees
%               freqLimits_norm : zone frequentielle de linearite du spectre log-log
% --------------------
% En sortie :  log_Sx : spectre en echelle log
%               theta_ini : initialisation du vecteur theta a estimer
%               (gamma, alpha, sigma2)
%               tab_log_fnorm : frequences normalisees en echelle log
% --------------------
function [log_Sx, theta_ini, tab_log_fnorm] = estime_pre_process(SpectreSLA,freq_norm,freqLimits_norm)
%estime_pre_process compute variables for fitting function
%   
tab_log_f = log(freq_norm);  %Echelle de frequence logarithmique normalisee
ind_linreg =  get_index_norm(freqLimits_norm,tab_log_f);

H = [ones(length(ind_linreg),1) tab_log_f(ind_linreg)'];

Sx = SpectreSLA';
log_Sx = log(Sx);
%Linear regression of log(S(f)) over [fnorm_begin_linreg,fnorm_end_linreg]
coef_linreg = H\log_Sx(ind_linreg);

%Initial estimates of [gamma alpha sigma2]
alpha_ini = -coef_linreg(2);
sigma2_ini = mean( Sx(ind_linreg(end):end) );
gamma_ini = mean( Sx(1:ind_linreg(1)) ) / sigma2_ini - 1;
theta_ini = [gamma_ini alpha_ini sigma2_ini]';
tab_log_fnorm = tab_log_f';
end

