% December 2019
% Developed by TeSA and CNES (France, Toulouse)
% contact
%   corinne.mailhes@tesa.prd.fr
%   amandine.guillot@cnes.fr
%
% Description : simulated SLA generation
% --------------------
% Fonctions appelees (en dehors des fonctions matlab) :
%      aucune
% --------------------
% En entree :
    % NbPts = nombre d'echantillons generes
    % Fmin = debut de la pente en km-1
    % Fcross = fin de la pente en km-1 - croisement avec sigma2
    % alpha = pente 
    % plateaubruit = niveau du plateau de bruit lu sur le spectre
        % Smax----Fmin
        %          \
        %           \
        %          Fcross-----plateaubruit
    % Fe = frequence d'echantillonnage en km-1
    % methode = 1 methode par correlation
    %         = 2 methode par TF directe
    %         = 3 methode par TF bruitee (voir rapport pour differences entre
    %         ces methodes
% --------------------
% En sortie :
    % signal = signal genere
    % G_true = pour generer directement d'autres echantillons (methode = 1)
    % Sx_true et f_pos : spectre theorique et le vecteur de frequences
    % associe
% --------------------


function [signal,G_true, Sx_true, f_pos] = generation_signal_simule_SLA(NbPts,Fmin,Fcross,alpha,plateaubruit,Fe,methode)

%+++++ THEORETICAL PSD +++++
    nfft = 2^14;    %number of Fourier bins (very large to have a fine grid)
    f_norm = (-0.5:1/nfft:0.5-1/nfft).'; % en frequences normalisees
    F = f_norm * Fe; % en km-1 (va de -Fe/2...0...Fe/2)
    ind_freq_pos = find(F > 0); % contient les indices des frequences positives
    f_pos = F(ind_freq_pos); % valeurs de frequences positives
    Sx_true = zeros(length(f_norm),1);
    %High frequencies
    ind_high = find(abs(F) >= Fmin); % donne les indices de -Fe/2,...,-Fmin, Fmin,...,Fe/2
    Sx_true(ind_high) = abs(F(ind_high)).^(-alpha); % pour ces indices, calcul de la pente
    % crossing spectral points between the slope and the noise level
    ind_highend = min(find((F) >= Fcross)); % correspond au point de croisement (freq positives)
    Sx_true(ind_high) = Sx_true(ind_high) / Sx_true(ind_highend) *plateaubruit;
    %Very low frequencies
    ind_low = find(abs(F) < Fmin);
    ind_low_nonzero = ind_low(ind_low ~= nfft/2+1);
    Sx_true(ind_low_nonzero) = Sx_true(max(ind_low_nonzero)+1) * ones(length(ind_low_nonzero),1);
    G_true = 0; % pour les methodes 2 et 3
    
    if methode ==1,
        %Correlation function
        rxx_true = ifft([Sx_true(nfft/2+1:nfft)*Fe/2; Sx_true(1:nfft/2)*Fe/2]);
        rxx_true = fftshift(real(rxx_true));
        % plot(imag(rxx_true)); pause; % verif c'est bien nul - ok
        power_true = rxx_true(nfft/2+1);
        %Signal to noise ratio
        Sx_true = Sx_true + plateaubruit;
        rxx_true(nfft/2+1) = rxx_true(nfft/2+1) + plateaubruit*Fe/2;
        %Correlation matrix
        R_true = toeplitz(rxx_true(nfft/2+1:nfft/2+NbPts));
        %Cholesky factor
        G_true = chol(R_true,'lower');

        signal = G_true * randn(NbPts,1);
    end
    
    if methode == 2 | methode ==3,
        Sx_sig = Sx_true; % sans bruit
        Sx_true = Sx_true + plateaubruit; % avec le plateau de bruit
        if methode ==2,
            module = sqrt([Sx_true(nfft/2+1:nfft); Sx_true(1:nfft/2)]); % explications voir 2017-AnalyseSpectraleSLA-addiitif.pptx
        else
            module = sqrt([Sx_sig(nfft/2+1:nfft); Sx_sig(1:nfft/2)]); % explications voir 2017-AnalyseSpectraleSLA-addiitif.pptx
        end
        phase = exp(1i*(rand(size(module))*2*pi));
        TFsignal = module .* phase;
        % symetrie
        TFsignal(nfft/2 +2 : nfft) = TFsignal(nfft/2 : -1 : 2)';
        TFsignal(nfft/2 +1) = 0; % pour que la TFinverse soit reelle       
        signaltotal = real (ifft (TFsignal,nfft));
        % figure; plot(imag(ifft(TFsignal,nfft))); pause; % verif c'est bien nul - ok  
        signal = signaltotal(1:NbPts)*sqrt(nfft* Fe/2); % explications voir 2017-AnalyseSpectraleSLA-addiitif.pptx
       
        if methode == 3, % rajout du bruit
            signal = signal + randn(size(signal))*sqrt(plateaubruit*Fe/2);
        end
    end
  
    % pour tracer facilement, pour coller a f_pos
    Sx_true = Sx_true(nfft/2+2:nfft);
    